#!/usr/bin/python
# -*- coding: ascii -*-
###########################################################################
# Terminmelder v1.03
# von REXX nach Python umgesetzt am 17.03.2012
# REXX-Version vom 27.05.1995 - 09.06.1995 (v1.02)
###########################################################################
# Das Format der Termindatei :
# Leere Zeilen und Zeilen, die mit einem Semikolon beginnen, werden
# ignoriert. Alle anderen Zeilen enthalten Termine. Diese Zeilen
# werden in der Form :
# Datum +Tage Nachricht
# erwartet. Also z.B.:
# 27.05.1995  7  Dies ist der Meldungstext
# 
# Mit diesem Eintrag wuerde vom 20.05.1995 bis zum 27.05.1995 (incl.) der
# Meldungstext ausgegeben werden.
# 
# Soll eine Altersberechnung bei Geburtstagen durchgefuehrt werden, so
# ist das Geburtsjahr (Angabe des Jahrhunderts ist hierbei zwingend
# noetig!) in Klammern zu setzen. Im Ausgabetext wird die
# Zeichenfolge <alter> durch das berechnete Alter ersetzt.
# Beispielsweise wird durch die Zeile
# 11.6.(1967) 7 Ich werde <alter> Jahre alt
# vom 4.6.1995 - 11.6.1995 der Text "Ich werde 28 Jahre alt" ausgegeben.
#
# Datum (1) : T.M.J fuer einmalige Termine
# Datum (2) : T.M   fuer jaehrlich wiederkehrende Termine
# Datum (3) : T     fuer monatlich wiederkehrende Termine
# Datum (4) : W     fuer woechentlich wiederkehrende Termine
#   T : 1 - 31
#   M : 1 - 12
#   J : 0 - ....
#   W : mo | di | .... | so
#
# +Tage     : Anzahl der Tage vor dem Termin, ab dem der
#             Nachrichtentext bereits ausgegeben werden soll.
#             Diese Zahl muss kleiner 365 sein!
#             Es koennen auch mehrere absolute Zeitpunkte angegeben
#             werden, an denen schon vorgewarnt wird.
#             Ein 30,14,7,3 warnt 30, 14 und 7 Tage vor dem Termin
#             und ab drei Tage vor dem Termin wird taeglich gewarnt.
#             Leerzeichen zwischen den einzelnen Werten sind nicht erlaubt!
#
# Nachricht : der anzuzeigende Text
# 
###########################################################################
import os
import sys
import codecs
import locale
import datetime
from Tkinter import *

fn = "/home/dede/daten/Termine.dat"


FileEncoding = "ISO-8859-15"
#      0         1           2           3             4          5          6
tage=["Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag", "Sonntag"]
tage_k=["mo", "di", "mi", "do", "fr", "sa", "so"]
d=datetime.date.today()
dow=d.weekday()

fl = codecs.open(fn, 'r', FileEncoding).readlines()

def IncMonat(datum):
  if datum.month<12:
    datum=datetime.date(datum.year, datum.month+1, datum.day)
  else:
    datum=datetime.date(datum.year+1, 1, datum.day)
  return datum

def IncJahr(datum):
  return datetime.date(datum.year+1, datum.month, datum.day)


meldung=[]

for line in fl:
  lns=line.replace("\t", " ").strip()
  if not (lns=="" or lns[0]==";"):
    sl=lns.split(" ", 1) # jede Zeile in seine drei Komponenten zerlegen (datum, warnbereich, meldung)
    datum=sl[0]
    sl=sl[1].strip().split(" ", 1)
    warn=sl[0]
    msg=sl[1].strip()

    z=datum.split(".") # jedes Datum zerlegen (tag, monat, jahr / geburtstag)
    tag=z[0].lower()
    if len(z)>1:
      monat=z[1]
    else:
      monat=""
    if len(z)>2:
      jahr=z[2]
      if jahr.startswith("("):
        geburtstag=str(jahr.strip("()"))
        jahr=""
    else:
      jahr=""
      geburtstag="0"

    if tage_k.count(tag)>0: # wenn Tag als Wochentag angegeben wurde, in Datum umsetzen
      if monat!="" or jahr!="":
        meldung.append("Fehler in Zeile: " + lns)
        continue
      idx_t=tage_k.index(tag)
      dif=((idx_t-dow)+7)%7
      tag=str((d+datetime.timedelta(dif)).day) # von "heute" bis "in maximal sechs Tagen"

    rng3=warn.split(",")
    rng=rng3.pop()

    flg=0  # alle Variablen bezueglich ihres Datentyps pruefen
    for i in rng3:
      if not i.isdigit():
        flg=1
    if (not tag.isdigit()) or (not rng.isdigit()) or \
       ((not monat.isdigit()) and monat!="") or \
       ((not jahr.isdigit()) and jahr!="") or \
       ((not geburtstag.isdigit()) and geburtstag!="") or \
       msg=="" or flg==1:
      meldung.append("Fehler in Zeile: " + lns)
      continue

    flg=0 # alle Variablen bzgl. ihrer Wertebereiche pruefen
    for i in rng3:
      if int(i)<0 or int(i)>365:
        flg=1
    if int(tag)<1 or int(tag)>31 or \
       (monat!="" and (int(monat)<1 or int(monat)>12)) or int(rng)<1 or int(rng)>365 or flg==1:
      meldung.append("Fehler in Zeile: " + lns)
      continue

    termin="" # termin wird nun auf ein vollstaendiges Datum mit Tag, Monat, Jahr ergaenzt
    if jahr=="":
      if monat=="": # nur Tag angegeben
        termin=datetime.date(d.year, d.month, int(tag)) # mit aktuellem Monat und Jahr auffuellen
        if termin<d: # wenn der Termin schon vorbei ist...
          termin2=IncMonat(termin) # ...in den naechsten Monat wechseln

          if termin2<d: # wenn es dabei zu einem Jahresueberlauf kam...
            termin=IncJahr(termin) #...ins naechste Jahr wechseln
          else:
            termin=termin2
      else: # Tag und Monat angegeben
        termin=datetime.date(d.year, int(monat), int(tag)) # mit Jahr auffuellen
        if termin<d: # wenn es dabei zu einem Jahresueberlauf kam...
          termin=IncJahr(termin) #...ins naechste Jahr wechseln
    else: # vollstaendiges Datum angegeben
      termin=datetime.date(int(jahr), int(monat), int(tag))

    flg=0
    if d>=(termin-datetime.timedelta(int(rng))) and d<=termin: # wenn Datum im untersten Tagesbereich...
      flg=1
    for i in rng3:
      if termin-datetime.timedelta(int(i))==d: # ...oder genau auf einen hoeheren Tageswert liegt
        flg=1 # ...dann ist das zu melden

    if flg==1: # Datum ist zu melden
      dif=termin-d # Differnz an Tagen zwischen Termin und Tagesdatum
      dt=str(termin.day) + "." + str(termin.month) + "." + str(termin.year) # Termin formatiert
      tg=tage[termin.weekday()] # Wochentag des Termins

      if int(geburtstag)!=0: # bei Geburtstags-Termin...
        alter=termin.year-int(geburtstag) #...das Alter berechnen
        msg=msg.replace("<alter>", str(alter)) #...und in den Meldungstext einbauen

      if dif==datetime.timedelta(0): # wenn es ein genauer Termin-Treffer ist...
         meldung.append("heute ist " + tg + " der " + dt + "   -   " + msg + "\n")
      else: # ansonsten ist zusaetzlich die Tages-Differenz auszugeben...
         meldung.append("noch " + str(dif.days) + " Tag[e] bis zum " + str(tg) + " den " + dt + "   -   " + msg + "\n")


def ende(egal):
  sys.exit()

if meldung!=[]: # wenn es etwas zu melden gibt (kann auch nur eine Fehlermeldung sein)
  root=Tk()
  strg=""
  for i in meldung: # zeilenweise Ausgabe
    strg+="\n" + i

  root.title("Terminmelder")
  lab=Label(root,text=strg)
  lab.pack()
  but=Button(root, text="OK", command=root.quit, default="normal", anchor="s", underline=0)
  but.focus()
  but.bind('<Return>', ende)
  but.bind('<Escape>', ende)
  but.pack()
  root.geometry("+%d+%d" % (500, 500)) # damit es bei meinen beiden Monitoren etwa in der Mitte von Schirm(1) landet
  root.mainloop()


